### Name: sp.predict
### Title: Prediction for new points given a ggt.sp, sp.lm, or
###   bayes.geostat.exact object
### Aliases: sp.predict
### Keywords: misc

### ** Examples

## Not run: 
##D ##Portions of this example requires MBA package to make surfaces
##D library(MBA)
##D 
##D ###########################################
##D ##  Prediction for bayes.geostat.exact
##D ###########################################
##D data(FBC07.dat)
##D Y <- FBC07.dat[1:150,"Y.2"]
##D coords <- as.matrix(FBC07.dat[1:150,c("coord.X", "coord.Y")])
##D 
##D n.samples <-1000
##D n = length(Y)
##D p = 1
##D 
##D phi <- 0.15
##D nu <- 0.5
##D 
##D beta.prior.mean <- as.matrix(rep(0, times=p))
##D beta.prior.precision <- matrix(0, nrow=p, ncol=p)
##D 
##D alpha <- 5/5
##D 
##D sigma.sq.prior.shape <- 2.0
##D sigma.sq.prior.rate <- 5.0
##D 
##D ##############################
##D ##Simple linear model with
##D ##the default exponential
##D ##spatial decay function
##D ##############################
##D m.1 <- bayes.geostat.exact(Y~1, n.samples=n.samples,
##D                            beta.prior.mean=beta.prior.mean,
##D                            beta.prior.precision=beta.prior.precision,
##D                            coords=coords, phi=phi, alpha=alpha,
##D                            sigma.sq.prior.shape=sigma.sq.prior.shape,
##D                            sigma.sq.prior.rate=sigma.sq.prior.rate,
##D                            sp.effects=TRUE)
##D 
##D ##Now prediction
##D set.seed(1)
##D pred.coords <- expand.grid(seq(0,100,length=10),seq(0,100,length=10))
##D pred.covars <- as.matrix(rep(1,nrow(pred.coords)))
##D 
##D m.1.pred <- sp.predict(m.1, pred.coords=pred.coords,
##D                        pred.covars=pred.covars, thin=5)
##D 
##D par(mfrow=c(2,2))
##D obs.surf <-
##D   mba.surf(cbind(coords, Y), no.X=100, no.Y=100, extend=T)$xyz.est
##D image(obs.surf, xaxs = "r", yaxs = "r", main="Observed response")
##D points(coords, pch=19, cex=1, col="green")
##D contour(obs.surf, add=T)
##D 
##D w.hat <- rowMeans(m.1$sp.effects)
##D w.surf <-
##D   mba.surf(cbind(coords, w.hat), no.X=100, no.Y=100, extend=T)$xyz.est
##D image(w.surf, xaxs = "r", yaxs = "r", main="Random effects")
##D points(coords, pch=19, cex=1, col="green")
##D contour(w.surf, add=T)
##D 
##D y.hat <- rowMeans(m.1.pred)
##D y.surf <-
##D   mba.surf(cbind(pred.coords, y.hat), no.X=100, no.Y=100, extend=T)$xyz.est
##D image(y.surf, xaxs = "r", yaxs = "r", main="Predicted response")
##D points(pred.coords, pch=19, cex=1, col="black")
##D rect(0, 0, 50, 50, col=NA, border="green")
##D contour(y.surf, add=T)
##D 
##D y.var <- apply(m.1.pred, 1, var)
##D y.surf <-
##D   mba.surf(cbind(pred.coords, y.var), no.X=100, no.Y=100, extend=T)$xyz.est
##D image(y.surf, xaxs = "r", yaxs = "r", main="Predicted response\nvariance")
##D points(coords, pch=19, cex=1, col="green")
##D points(pred.coords, pch=19, cex=1, col="black")
##D rect(0, 0, 50, 50, col=NA, border="green")
##D contour(y.surf, add=T)
##D  
##D ###########################################
##D ##       Prediction for sp.lm
##D ###########################################
##D data(rf.n200.dat)
##D 
##D Y <- rf.n200.dat$Y
##D coords <- as.matrix(rf.n200.dat[,c("x.coords","y.coords")])
##D w <- rf.n200.dat$w
##D 
##D pred.coords <- expand.grid(seq(1,10,1), seq(1,10,1))
##D n.pred <- nrow(pred.coords)
##D 
##D ###############################
##D ##Prediction with a sp.lm model
##D ###############################
##D m.2 <- sp.lm(Y~1, coords=coords,
##D              starting=list("phi"=0.6,"sigma.sq"=1, "tau.sq"=1),
##D              sp.tuning=list("phi"=0.01, "sigma.sq"=0.05, "tau.sq"=0.05),
##D              priors=list("phi.Unif"=c(0.3, 3), "sigma.sq.IG"=c(2, 1),
##D                "tau.sq.IG"=c(2, 1)),
##D              cov.model="exponential",
##D              n.samples=1000, verbose=TRUE, n.report=100, sp.effects=TRUE)
##D 
##D pred <- sp.predict(m.2, pred.coords,
##D                    pred.covars=as.matrix(rep(1,n.pred)))
##D 
##D par(mfrow=c(1,2))
##D obs.surf <-
##D   mba.surf(cbind(coords, Y), no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D image(obs.surf, xaxs = "r", yaxs = "r", main="Observed response")
##D points(coords)
##D contour(obs.surf, add=T)
##D 
##D y.hat <- rowMeans(pred$y.pred)
##D y.pred.surf <-
##D   mba.surf(cbind(pred.coords, y.hat), no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D image(y.pred.surf, xaxs = "r", yaxs = "r", main="Predicted response")
##D points(coords, pch=1, cex=1)
##D points(pred.coords, pch=19, cex=1)
##D contour(y.pred.surf, add=T)
##D legend(1.5,2.5, legend=c("Obs.", "Pred."), pch=c(1,19),
##D        cex=c(1,1), bg="white")
##D 
##D ###############################
##D ##Prediction with a sp.lm
##D ##predictive process model
##D ###############################
##D m.3 <- sp.lm(Y~1, coords=coords, knots=c(6,6,0),
##D              starting=list("phi"=0.6,"sigma.sq"=1, "tau.sq"=1),
##D              sp.tuning=list("phi"=0.01, "sigma.sq"=0.01, "tau.sq"=0.01),
##D              priors=list("phi.Unif"=c(0.3, 3), "sigma.sq.IG"=c(2, 1),
##D                "tau.sq.IG"=c(2, 1)),
##D              cov.model="exponential",
##D              n.samples=2000, verbose=TRUE, n.report=100, sp.effects=TRUE)
##D 
##D print(summary(m.3$p.samples))
##D plot(m.3$p.samples)
##D 
##D pred <- sp.predict(m.3, pred.coords,
##D                    pred.covars=as.matrix(rep(1,n.pred)))
##D 
##D par(mfrow=c(1,2))
##D obs.surf <-
##D   mba.surf(cbind(coords, Y), no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D image(obs.surf, xaxs = "r", yaxs = "r", main="Observed response")
##D points(coords)
##D contour(obs.surf, add=T)
##D 
##D y.hat <- rowMeans(pred$y.pred)
##D y.pred.surf <-
##D   mba.surf(cbind(pred.coords, y.hat), no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D image(y.pred.surf, xaxs = "r", yaxs = "r", main="Predicted response")
##D points(coords, pch=1, cex=1)
##D points(m.3$knot.coords, pch=3, cex=1)
##D points(pred.coords, pch=19, cex=1)
##D contour(y.pred.surf, add=T)
##D legend(1.5,2.5, legend=c("Obs.", "Knots", "Pred."),
##D        pch=c(1,3,19), cex=c(1,1,1), bg="white")
##D 
##D ###########################################
##D ##       Prediction for ggt.sp
##D ###########################################
##D data(FBC07.dat)
##D 
##D ##Divide the data into model and prediction sets
##D Y.1 <- FBC07.dat[1:100,"Y.1"]
##D Y.2 <- FBC07.dat[1:100,"Y.2"]
##D model.coords <- as.matrix(FBC07.dat[1:100,c("coord.X", "coord.Y")])
##D pred.coords <- as.matrix(FBC07.dat[151:200,c("coord.X", "coord.Y")])
##D 
##D #############################
##D ##   Univariate model
##D #############################
##D 
##D ##Fit some model with ggt.sp.
##D K.prior <- prior(dist="IG", shape=2, scale=5)
##D Psi.prior <- prior(dist="IG", shape=2, scale=5)
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D var.update.control <-
##D   list("K"=list(starting=5, tuning=0.5, prior=K.prior),
##D        "Psi"=list(starting=5, tuning=0.5, prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5, prior=phi.prior)
##D        )
##D 
##D beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))
##D 
##D run.control <- list("n.samples"=1000)
##D 
##D Fit <- ggt.sp(formula=Y.2~1, run.control=run.control,
##D               coords=model.coords,
##D               var.update.control=var.update.control,
##D               beta.update.control=beta.control,
##D               cov.model="exponential")
##D 
##D ##Now make predictions for the holdout set.
##D ##Step 1. make the design matrix for the prediction points.
##D pred.covars <- as.matrix(rep(1, nrow(pred.coords)))
##D 
##D ##Step 2. call sp.predict.
##D Pred <- sp.predict(Fit, pred.covars=pred.covars,
##D                    pred.coords=pred.coords)
##D 
##D ##Step 3. check out the predicted random effects and
##D ##predicted response variable.
##D 
##D Pred.sp.effects.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred$pred.sp.effects)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D Pred.Y.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred$pred.y)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D par(mfrow=c(1,2))
##D image(Pred.sp.effects.surf, xaxs="r", yaxs="r",
##D       main="Predicted random spatial effects")
##D contour(Pred.sp.effects.surf, add=TRUE)
##D 
##D image(Pred.Y.surf, xaxs="r", yaxs="r",
##D       main="Predicted Y.2")
##D contour(Pred.Y.surf, add=TRUE)
##D 
##D #############################
##D ##   Multivariate models
##D #############################
##D 
##D ##Fit some model with ggt.sp.
##D K.prior <- prior(dist="IWISH", df=2, S=diag(c(3, 6)))
##D Psi.prior <- prior(dist="IWISH", df=2, S=diag(c(7, 5)))
##D phi.prior <- prior(dist="UNIF", a=0.06, b=3)
##D 
##D K.starting <- matrix(c(2,-3, 0, 1), 2, 2)
##D Psi.starting <- diag(c(3, 2))
##D 
##D var.update.control <-
##D   list("K"=list(starting=K.starting, tuning=diag(c(0.1, 0.5, 0.1)),
##D          prior=K.prior),
##D        "Psi"=list(starting=Psi.starting, tuning=diag(c(0.1, 0.5, 0.1)),
##D          prior=Psi.prior),
##D        "phi"=list(starting=0.1, tuning=0.5,
##D          prior=list(phi.prior, phi.prior))
##D        )
##D 
##D beta.control <- list(update="GIBBS", prior=prior(dist="FLAT"))
##D 
##D run.control <- list("n.samples"=1000, "sp.effects"=FALSE)
##D 
##D Fit.mv <-
##D   ggt.sp(formula=list(Y.1~1, Y.2~1), run.control=run.control,
##D          coords=model.coords,
##D          var.update.control=var.update.control,
##D          beta.update.control=beta.control,
##D          cov.model="exponential")
##D 
##D ##Now make predictions for the holdout set.
##D ##Step 1. make the design matrix for the prediction points using
##D ##the mk.mv.X function.
##D pred.covars.1 <- as.matrix(rep(1, nrow(pred.coords)))
##D pred.covars.2 <- as.matrix(rep(1, nrow(pred.coords)))
##D 
##D pred.covars.mv <- mk.mv.X(list(pred.covars.1, pred.covars.2))
##D 
##D ##Step 2. call sp.predict.
##D Pred.mv <- sp.predict(Fit.mv, pred.covars=pred.covars.mv,
##D                       pred.coords=pred.coords)
##D 
##D ##Step 3. check out the predicted random effects and
##D ##predicted response variables.   Recall, these are
##D ##organized as m consecutive rows for each point.
##D Pred.sp.effects.1 <-
##D   Pred.mv$pred.sp.effects[seq(1, nrow(Pred.mv$pred.sp.effects), 2),]
##D 
##D Pred.sp.effects.2 <-
##D   Pred.mv$pred.sp.effects[seq(2, nrow(Pred.mv$pred.sp.effects), 2),]
##D 
##D Pred.Y.1 <-
##D   Pred.mv$pred.sp.effects[seq(1, nrow(Pred.mv$pred.y), 2),]
##D 
##D Pred.Y.2 <-
##D   Pred.mv$pred.sp.effects[seq(2, nrow(Pred.mv$pred.y), 2),]
##D 
##D ##Then map.
##D Pred.sp.effects.1.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred.sp.effects.1)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D Pred.sp.effects.2.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred.sp.effects.2)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D Pred.Y.1.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred.Y.1)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D Pred.Y.2.surf <-
##D   mba.surf(cbind(pred.coords, rowMeans(Pred.Y.2)),
##D            no.X=100, no.Y=100, extend=TRUE)$xyz.est
##D 
##D par(mfrow=c(2,2))
##D image(Pred.sp.effects.surf, xaxs="r", yaxs="r",
##D       main="Predicted random spatial effects Y.1")
##D contour(Pred.sp.effects.1.surf, add=TRUE)
##D 
##D image(Pred.sp.effects.surf, xaxs="r", yaxs="r",
##D       main="Predicted random spatial effects Y.2")
##D contour(Pred.sp.effects.2.surf, add=TRUE)
##D 
##D image(Pred.sp.effects.surf, xaxs="r", yaxs="r",
##D       main="Predicted Y.1")
##D contour(Pred.Y.1.surf, add=TRUE)
##D 
##D image(Pred.sp.effects.surf, xaxs="r", yaxs="r",
##D       main="Predicted Y.2")
##D contour(Pred.Y.2.surf, add=TRUE)
##D 
## End(Not run)



